@@ -3,6 +3,8 @@ use rusoto_s3::{S3, PutObjectRequest};
 use futures::compat::Future01CompatExt;
 use std::error::Error;
 use std::time::Duration;
+use crate::retry::RetryHandler;
+use futures_retry::{RetryPolicy, FutureRetry, StreamRetryExt};
 
 #[async_trait]
 pub trait EventEmitter {
@@ -19,7 +21,7 @@ pub struct S3EventEmitter<S, F>
 {
     s3: S,
     output_bucket: String,
-    key_fn: F
+    key_fn: F,
 }
 
 impl<S, F> S3EventEmitter<S, F>
@@ -51,11 +53,14 @@ impl<S, F> EventEmitter for S3EventEmitter<S, F>
         self.s3.put_object(
             PutObjectRequest {
                 body: Some(event.into()),
-                bucket: "".to_string(),
+                bucket: self.output_bucket.clone(),
                 key,
                 ..Default::default()
             }
-        ).with_timeout(Duration::from_secs(2)).compat().await?;
+        )
+            .with_timeout(Duration::from_secs(2))
+            .compat()
+            .await?;
 
         Ok(())
     }
